/*
bhhardware.cpp
Copyright (C)2004 Al Kingsley

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   Don't write to me at: kingsley@bananahead.com 

   Part of:
   BhPos, Copyright (C) 2001-2004 Al Kingsley
   BhPos comes with ABSOLUTELY NO WARRANTY.
*/
#include <dlfcn.h>
#include <dirent.h>
#include <cstdio>
#include "bhhardware.h"

#define _DEBUGGERY

namespace bhpos{

CHardwareManager::CHardwareManager()
{
   barcode=NULL;
   printer=NULL;
   cashdrawer=NULL;
   custdisp=NULL;
   magswipe=NULL;
   portable=NULL;
   scales=NULL;
}

CHardwareManager::CHardwareManager(CHardwareManager const& mc)
{
   barcode=mc.barcode;
   printer=mc.printer;
   cashdrawer=mc.cashdrawer;
   custdisp=mc.custdisp;
   magswipe=mc.magswipe;
   portable=mc.portable;
   scales=mc.scales;
}

void CHardwareManager::operator=(CHardwareManager const& mc)
{
   barcode=mc.barcode;
   printer=mc.printer;
   cashdrawer=mc.cashdrawer;
   custdisp=mc.custdisp;
   magswipe=mc.magswipe;
   scales=mc.scales;
}

bool CHardwareManager::operator==(CHardwareManager const& mc)
{
   return false;
}

bool CHardwareManager::operator<(CHardwareManager const& mc)
{
   return false;
}

CHardwareManager::~CHardwareManager()
{
   Close();
}

void CHardwareManager::Close()
{
}

bool CHardwareManager::Add(HardwareType t, BhDevType d)
{
   switch(t){
   case BHTYPE_BARCODE:
      barcode=new CBarcode(d);
      if(!barcode) return false;
      barcode->signal_read().connect(sigc::mem_fun(*this, &CHardwareManager::BarcodeMsg));
      break;
   case BHTYPE_PRINTER:
      printer=new CPrinter(d);
      if(!printer) return false;
      printer->signal_status().connect(sigc::mem_fun(*this, &CHardwareManager::printer_msg));
      printer->signal_error().connect(sigc::mem_fun(*this, &CHardwareManager::printer_errormsg));
      break;
   case BHTYPE_CASHDRAWER:
      cashdrawer=new CCashD(d);
      break;
   case BHTYPE_CUSTDISP:
      custdisp=new CCustDisp(d);
      break;
   case BHTYPE_PORTABLE:
      portable=new CPortable(d);
      break;
   case BHTYPE_SCALES:
      scales=new CScales(d);
      break;
   }
   return true;
}

bool CHardwareManager::Remove(HardwareType t)
{
   return true;
}

int CHardwareManager::SetOption(HardwareType t, HardwareOption o, void *d)
{
   switch(t){
   case BHTYPE_BARCODE:
   case BHTYPE_PRINTER:
   case BHTYPE_CASHDRAWER:
   case BHTYPE_CUSTDISP:
   case BHTYPE_PORTABLE:
   case BHTYPE_MAGSWIPE:
   case BHTYPE_SCALES:
      break;
   }
}

CBhDevice *CHardwareManager::GetDevice(HardwareType t)
{
   switch(t){
   case BHTYPE_BARCODE: return dynamic_cast<CBhDevice *>(barcode);
   case BHTYPE_PRINTER: return dynamic_cast<CBhDevice *>(printer);
   case BHTYPE_CASHDRAWER: return dynamic_cast<CBhDevice *>(cashdrawer);
   case BHTYPE_CUSTDISP: return dynamic_cast<CBhDevice *>(custdisp);
   case BHTYPE_PORTABLE: return dynamic_cast<CBhDevice *>(portable);
   case BHTYPE_MAGSWIPE: return dynamic_cast<CBhDevice *>(magswipe);
   case BHTYPE_SCALES: return dynamic_cast<CBhDevice *>(scales);
      break;
   }
}
/*
int CHardwareManager::Setup(CMfPosEngine *eng)
{
   int devtype;
   bhpos::BarcodeDeviceType    dBcscanner;
   bhpos::CashdrawerDeviceType dCashd;
   bhpos::CustdispDeviceType   dCustdisp;
   bhpos::PrinterDeviceType    dPrinter;
   bhpos::MagSwipeDeviceType   dMagswipe;
   bhpos::ScalesDeviceType     dScales;
   
   eng->GetOption(MFENG_CONFIG_BCSCANNER_DEVTYPE, &dBcscanner);
   eng->GetOption(MFENG_CONFIG_CASHDRAWER_DEVTYPE, &devtype);
   eng->GetOption(MFENG_CONFIG_PRINTER_DEVTYPE, &devtype);
   eng->GetOption(MFENG_CONFIG_CUSTDISP_DEVTYPE, &devtype);
   eng->GetOption(MFENG_CONFIG_MAGSWIPE_DEVTYPE, &devtype);
   eng->GetOption(MFENG_CONFIG_SCALES_DEVTYPE, &devtype);
}
*/
int CHardwareManager::Process(int t, void *d)
{
   return 0;
}

void CHardwareManager::BarcodeMsg(int)
{
   CBhDevice *dev;
      
   dev=dynamic_cast<CBhDevice *>(barcode);
   
   char       data[256];
   int        len;
   
   if(dev){
      //len=dev.Read(0, 255, data);
      hw_signal.emit(BHTYPE_BARCODE, len, data);
   }
}

void CHardwareManager::CashDMsg(int)
{
}

void CHardwareManager::CustDispMsg(int)
{
}

void CHardwareManager::PrinterMsg(int)
{
}

void CHardwareManager::MagswipeMsg(int)
{
}

void CHardwareManager::ScalesMsg(int)
{
}

void CHardwareManager::printer_msg(BhPrinterStatusEvent t, void *d)
{
   hw_signal.emit(BHTYPE_PRINTER, t, d);
}

void CHardwareManager::printer_errormsg(BhPrinterErrorEvent t, void *d)
{
   hwerror_signal.emit(BHTYPE_PRINTER, t, d);
}


}
