/*
bhhardware.h
Copyright (C)2004 Al Kingsley

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Don't write to me at: kingsley@bananahead.com 

	Part of:
	BhPos, Copyright (C) 2001-2005 Al Kingsley
	BhPos comes with ABSOLUTELY NO WARRANTY.
*/
#ifndef _BHHARDWARE_H_
#define _BHHARDWARE_H_

#ifndef _BHQUEUE_T_
#include "pos.h" 
#endif // _BHQUEUE_T_
#include <bhdev.h>
#include <hw/bhbarcode.h>
#include <hw/bhcashd.h>
#include <hw/bhprinter.h>
#include <hw/bhcustdisp.h>
#include <hw/bhportable.h>
#include <hw/bhmagswipe.h>
#include <hw/bhscales.h>

#include <list>
#include <algorithm>

namespace bhpos{
	
/**
 * \defgroup hardware Plug-in Modules
 * \@{
 *
 * \ingroup mflib
 * \@{
 */
   
typedef union{
   unsigned char bmsg[4];
   unsigned int  imsg;
}bhm;
   
enum HardwareType{
   BHTYPE_BARCODE,
   BHTYPE_PRINTER,
   BHTYPE_CASHDRAWER,
   BHTYPE_CUSTDISP,
   BHTYPE_PORTABLE,
   BHTYPE_MAGSWIPE,
   BHTYPE_SCALES
};

enum HardwareOption{
   HWOPTION_DEVNAME
};

class CHardwareManager{
protected:
	CBarcode  *barcode;
   CPrinter  *printer;
   CCashD    *cashdrawer;
   CCustDisp *custdisp;
   CMagswipe *magswipe;
   CPortable *portable;
   CScales   *scales;

   void BarcodeMsg(int);
   void CashDMsg(int);
   void CustDispMsg(int);
   void PrinterMsg(int);
   void MagswipeMsg(int);
   void ScalesMsg(int);

	sigc::signal<void, HardwareType, int, void *> hw_signal;
   sigc::signal<void, HardwareType, int, void *> hwerror_signal;

   void printer_msg(BhPrinterStatusEvent, void *);
   void printer_errormsg(BhPrinterErrorEvent, void *);

public:
	CHardwareManager();
	CHardwareManager(CHardwareManager const&);
	~CHardwareManager();
	void operator=(CHardwareManager const&);
	bool operator==(CHardwareManager const&);
	bool operator<(CHardwareManager const&);
	
	bool  Add(HardwareType, BhDevType);
   bool  Remove(HardwareType);
	void  Close();
   int   SetOption(HardwareType, HardwareOption, void *);
   CBhDevice *GetDevice(HardwareType);
   int   Process(int, void *);
   sigc::signal<void, HardwareType, int, void *> signal_hw(){ return hw_signal; }
   sigc::signal<void, HardwareType, int, void *> signal_hwerror(){ return hwerror_signal; }
};


class CPluginHardwareManager{
protected:
	std::list<CBhDevice>	devices;
	
public:
	CPluginHardwareManager();
	CPluginHardwareManager(CPluginHardwareManager const&);
	~CPluginHardwareManager();
	void operator=(CPluginHardwareManager const&);
	bool operator==(CPluginHardwareManager const&);
	bool operator<(CPluginHardwareManager const&);
	
	bool Add(CBhDevice&);
   bool Remove(CBhDevice&);
	void Close();
   int  Process(int, void *);
};
/* @}
   @}
 */
}

#endif	//	_BHHARDWARE_H_
