/*
modstat.cpp
Copyright (C)2003  Al Kingsley


  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Don't write to me at: kingsley@bananahead.com 

	Part of:
	BhPos, Copyright (C) 2001-2003 Al Kingsley
	BhPos comes with ABSOLUTELY NO WARRANTY.

*/

#include "modstat.h"

namespace BhPos{

CModeStatus::CModeStatus()
{
   mode=BHCLIENT_MODE_UNKNOWN;
   status=BHCLIENT_STATUS_IDLE;
   keygrp=0;
}

CModeStatus::~CModeStatus()
{
}

CModeStatus::CModeStatus(BhClientMode m, BhClientStatus s, unsigned int k)
{
   mode=m;
   status=s;
   keygrp=k;
}

CModeStatus& CModeStatus::operator=(const CModeStatus& m)
{
   mode=m.mode;
   status=m.status;
   keygrp=m.keygrp;
   
   return *this;
}

CModeStatus* CModeStatus::operator&()
{
   return this;
}

bool CModeStatus::operator==(CModeStatus const& m)
{
   if(mode==m.mode &&
      status==m.status &&
      keygrp==m.keygrp)
      return true;
   
   return false;
}

bool CModeStatus::operator!=(CModeStatus const& m)
{
   if(mode!=m.mode ||
      status!=m.status ||
      keygrp!=m.keygrp)
      return true;
   
   return false;
}

bool CModeStatus::KeyEnabled(unsigned int kg)
{
   if(kg & keygrp) return true;
   return false;
}

void CModeStatus::Set(BhModStatParam p, void *v)
{
   switch(p){
   case BHMODSTAT_MODE:
      mode=*((BhClientMode *)v);
      break;
   
   case BHMODSTAT_STATUS:
      status=*((BhClientStatus *) v);
      break;
   
   case BHMODSTAT_KEYGROUP:
      keygrp=*((unsigned int *) v);
      break;
   }
}

void CModeStatus::Set(BhClientMode m, BhClientStatus s, unsigned int k)
{
   mode=m;
   status=s;
   keygrp=k;
}

int CModeStatus::Get(BhModStatParam p, void *v)
{
   switch(p){
   case BHMODSTAT_MODE: *((BhClientMode *) v)=mode;
   case BHMODSTAT_STATUS: *((BhClientStatus *) v)=status;
   case BHMODSTAT_KEYGROUP: *((unsigned int *) v)=keygrp;
   }
   
   return 0;
}

/////////////////////////////////////////////////////////////////////
//
//
//
//

CModeStatusList::CModeStatusList()
{
}   

CModeStatusList::~CModeStatusList()
{
}   

CModeStatusList& CModeStatusList::operator=(const CModeStatusList&)
{
   return *this;
}

CModeStatusList* CModeStatusList::operator&()
{
   return this;
}   

bool CModeStatusList::operator==(CModeStatusList const&)
{
   return false;
}   

bool CModeStatusList::operator!=(CModeStatusList const&)
{
   return false;
}   

void CModeStatusList::Clear()
{
   entry.clear();
}   

int  CModeStatusList::Push(const CModeStatus& ms)
{
   entry.push_back(ms);
   return entry.size();
}

bool CModeStatusList::Pop()
{
   if(entry.empty()) false;
	   
	entry.pop_back();
   if(entry.empty()) false;

   return true;
}

bool CModeStatusList::Pop(BhClientMode *m, BhClientStatus *s, unsigned int *kg)
{
   if(entry.empty()) false;
	
   CModeStatus ms;
   
	entry.pop_back();
   if(entry.empty()) false;
      
   ms=entry.back();
   ms.Get(BHMODSTAT_MODE, m);
   ms.Get(BHMODSTAT_STATUS, s);
   ms.Get(BHMODSTAT_KEYGROUP, kg);
   
   return true;
}

bool CModeStatusList::Current(BhClientMode *m, BhClientStatus *s, unsigned int *kg)
{
   if(entry.empty()) false;
	
   CModeStatus ms=entry.back();
   
   ms.Get(BHMODSTAT_MODE, m);
   ms.Get(BHMODSTAT_STATUS, s);
   ms.Get(BHMODSTAT_KEYGROUP, kg);
   
   return true;
}   

bool CModeStatusList::KeyEnabled(GdkEventKey *event)
{
   unsigned int kem=event->keyval;
   CModeStatus  ms=entry.back();

   ms.Get(BHMODSTAT_KEYGROUP, &kem);
   
	if(event->state & GDK_CONTROL_MASK){
      if(!(kem & KEY_ENABLED_CTRL)) return false;
	}
	
	if(event->state & GDK_SHIFT_MASK){
      if(!(kem & KEY_ENABLED_SHIFT)) return false;
	}
   /*
   if(event->state & GDK_ALT_MASK){
		if(!(kem & KEY_ENABLED_ALT)) return false;
	}
   */
	switch(event->keyval){
   case GDK_Escape:
		if(KEY_ENABLED_ESCAPE & kem) return true;
		return false;
		
	case GDK_KP_0:
	case GDK_KP_1:
	case GDK_KP_2:
	case GDK_KP_3:
	case GDK_KP_4:
	case GDK_KP_5:
	case GDK_KP_6:
	case GDK_KP_7:
	case GDK_KP_8:
	case GDK_KP_9:
	case GDK_0:
	case GDK_1:
	case GDK_2:
	case GDK_3:
	case GDK_4:
	case GDK_5:
	case GDK_6:
	case GDK_7:
	case GDK_8:
	case GDK_9:
	//case PK_NUMSEP:
	case PK_NUMDEC:
	case GDK_KP_Decimal:
		if(KEY_ENABLED_NUMERIC & kem) return true;
		return false;

	case GDK_KP_Equal:
	case GDK_KP_Add:
	case GDK_KP_Subtract:		
	case GDK_plus:
	case GDK_minus:
	case GDK_equal:
	   if(KEY_ENABLED_OPERATOR & kem)  return true;
		return false;
	
	case GDK_KP_Home:
	case GDK_KP_Up:
	case GDK_KP_Left:
	case GDK_KP_Right:
	case GDK_KP_Down:
	case GDK_KP_Page_Up:
	case GDK_KP_Page_Down:
	case GDK_KP_End:	
	case GDK_Left:
	case GDK_Right:
	case GDK_Up:
	case GDK_Down:
	case GDK_Home:
	case GDK_Page_Up:
	case GDK_Page_Down:
	case GDK_End:
	   if(KEY_ENABLED_NAVIGATION & kem) return true;
		return false;
	
	case GDK_KP_Enter:
	case GDK_Return:
	case GDK_Linefeed:
		if(KEY_ENABLED_RETURN & kem) return true;
		return false;
		
	case GDK_Delete:
	case GDK_KP_Delete:
	   if(KEY_ENABLED_DELETE & kem)  return true;
		return false;
	
	case GDK_Clear:
	case GDK_BackSpace:
		if(!(KEY_ENABLED_EDITING & kem)) return true;
		return false;
	
	case PK_REQUEST:
	case PK_PLULOOKUP:
		if(KEY_ENABLED_MISC & kem) return true;
		return false;

	case PK_QUANTITY:
		if(KEY_ENABLED_MAPPED & kem) return true;
		return false;
	
	case PK_CASH:
	case PK_CREDIT:
	case PK_CHEQUE:
//	case PK_SMARTCARD:
	case PK_CREDITRETURN:
	case PK_CREDITSLIP:
	case PK_GIFTVOUCHER:
	case PK_TPVOUCHER:
	case PK_INTCOUPON:
	case PK_TPCOUPON:
	case PK_LUNCHSTAMP:
	case PK_INHOUSECHARGE:
	case PK_STORECARD:
	case PK_OTHER:
	case PK_CREDIT_CB:
	case PK_CHEQUE_CB:
	case PK_SMARTC_CB:
	case PK_TOKEN_CB:
	case PK_VOUCHER_CB:
	case PK_COUPON_CB:
	case PK_OTHER_CB:
	   if(KEY_ENABLED_PAYTYPE & kem) return true;
		return false;
      
	case PK_RETURNS:
   case PK_SALETYPE:
      if(kem & KEY_ENABLED_PRESALE)  return true;
		return false;
      
   case PK_ABSWAP:
      if(kem & KEY_ENABLED_SWAP)  return true;
		return false;
      
   case PK_REMOVE:
      if(KEY_ENABLED_REMOVE & kem) return true;
		return false;
      
	case PK_PRINT:
		if(KEY_ENABLED_PRINT & kem) return true;
		return false;
		
	default:
		if(KEY_ENABLED_ALPHA & kem) return true;
		return false;
   }
}

}
