/*
bhcellrenderer.cpp
Copyright (C) 2004  Al Kingsley

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Part of:
	BhPos, Copyright (C) 2001-2004 Al Kingsley
	BhPos comes with ABSOLUTELY NO WARRANTY. 
*/
#include <gdk/gdkkeysyms.h>
#include <gtk/gtkentry.h>
#include "bhcellrenderer.h"

namespace bhpos{
   
namespace widget{

/***************************************************
 *
 * BhEntry
 *
 ***************************************************/
BhEntryEditable::BhEntryEditable(const Glib::ustring& path) : Glib::ObjectBase(typeid(BhEntryEditable)),
   Gtk::EventBox(), Gtk::CellEditable(),
   path_(path),
   entry(0),
   editing_canceled_(false)
{
   Gdk::Color col;

   entry = manage(new class Gtk::Entry());
   entry->set_has_frame(false);
   entry->gobj()->is_cell_renderer = true;
   set_flags(Gtk::CAN_FOCUS);
   add_events(Gdk::KEY_PRESS_MASK | Gdk::KEY_RELEASE_MASK);
   
   col.set_rgb(0xFFF, 0xFFF, 0xFFF);
   entry->modify_fg(Gtk::STATE_NORMAL, col);
   entry->modify_fg(Gtk::STATE_ACTIVE, col);
   entry->modify_fg(Gtk::STATE_SELECTED, col);
   col.set_rgb(0xFFFFFF, 0, 0);
   entry->modify_bg(Gtk::STATE_NORMAL, col);
   col.set_rgb(0, 0xFFFFFF, 0);
   entry->modify_bg(Gtk::STATE_ACTIVE, col);
   col.set_rgb(0, 0, 0xFFFFFF);
   entry->modify_bg(Gtk::STATE_SELECTED , col);
   show_all_children();
   add(*entry);
}

Glib::PropertyProxy_Base BhEntryEditable::_property_renderable()
{
   return _property_renderable();
}

Glib::ustring BhEntryEditable::get_path() const
{
  return path_;
}

void BhEntryEditable::set_text(const Glib::ustring& text)
{
   entry->set_text(text);
}

Glib::ustring BhEntryEditable::get_text() const
{
   return entry->get_text();
}

void BhEntryEditable::select_region(int start_pos, int end_pos)
{
   entry->select_region(start_pos, end_pos);
}
void BhEntryEditable::set_position(int p)
{
   entry->set_position(p);
}

int BhEntryEditable::get_text_length()
{
   if(!entry){
      g_print("no entry");
      return 0;
   }
   return entry->get_text_length();
}

bool BhEntryEditable::get_editing_canceled() const
{
  return editing_canceled_;
}

void BhEntryEditable::on_entry_activate()
{
  editing_done();
  //remove_widget(); // TODO: this line causes the widget to be removed twice -- dunno why
}

bool BhEntryEditable::on_key_press_event(GdkEventKey* event)
{
   g_print("blong\n");
   if(event->keyval == GDK_Escape){
      editing_canceled_ = true;

      editing_done();
      remove_widget();

      return true;
   }

   entry->grab_focus();

   // Hackish :/ Synthesize a key press event for the entry.

   GdkEvent synth_event;
   synth_event.key = *event;
   if(IsNumber(event)){
      synth_event.key.window = Glib::unwrap(entry->get_window()); // TODO: Use a C++ Gdk::Event.
      synth_event.key.send_event = true;
      entry->event(&synth_event);
      return Gtk::EventBox::on_key_press_event(event);
   }
   return true;
}

bool BhEntryEditable::on_entry_key_press_event(GdkEventKey* event)
{
   g_print("nob sock\n");
   if(event->keyval == GDK_Escape){
      editing_canceled_ = true;

      editing_done();
      remove_widget();

      return true;
   }

   return false;
}

bool BhEntryEditable::IsNumber(GdkEventKey *e)
{
   switch(e->keyval){
   case GDK_0:
   case GDK_1:
   case GDK_2:
   case GDK_3:
   case GDK_4:
   case GDK_5:
   case GDK_6:
   case GDK_7:
   case GDK_8:
   case GDK_9:
   case GDK_KP_0:
   case GDK_KP_1:
   case GDK_KP_2:
   case GDK_KP_3:
   case GDK_KP_4:
   case GDK_KP_5:
   case GDK_KP_6:
   case GDK_KP_7:
   case GDK_KP_8:
   case GDK_KP_9:
      return true;
   }
   return false;
}

/*
bool BhEntryEditable::on_focus_out_event(GdkEventFocus *event)
{
   bool result = entry->on_focus_out_event(event);
   //check_for_change();
   return result;
}*/
/*
void BhEntryEditable::set_editable(bool editable)
{
   entry->set_editable(editable);
}
void BhEntryEditable::set_text(Glib::ustring t)
{
    old_text = t;
   //Gtk::Entry::set_text(text);
   
   entry->set_text(t);
}
Glib::ustring BhEntryEditable::get_text()
{
   entry->get_text();
}
void BhEntryEditable::select_region(int s, int e)
{
   entry->select_region(s, e);
}
void BhEntryEditable::set_position(int p)
{
   entry->set_position(p);
}
*/

void BhEntryEditable::start_editing_vfunc(GdkEvent *event)
{
   int l=entry->get_text_length();
   g_print("kaboom! : %d\n", l);
   entry->grab_focus();
   entry->select_region(l, l);
   entry->set_position(l);
   
  // TODO: This is a key-binding signal. Investigate whether we really need to use a keybinding signal
  // when creating a derived CellRenderer.
  entry->signal_activate().connect(sigc::mem_fun(*this, &Self::on_entry_activate));
  entry->signal_key_press_event().connect(sigc::mem_fun(*this, &Self::on_entry_key_press_event));
   /*
   entry->start_editing(event);
   entry->grab_focus();
   entry->select_region(0, 0);
   entry->set_position(-1);
  
   select(type){
   case BHCELLCONTENTTYPE_TEXT:
   case BHCELLCONTENTTYPE_INTEGER:
   case BHCELLCONTENTTYPE_CURRENCY:
   case BHCELLCONTENTTYPE_FLOAT:
   case BHCELLCONTENTTYPE_COMBO:
   case BHCELLCONTENTTYPE_PROGRESS:
   case BHCELLCONTENTTYPE_DATE:
   case BHCELLCONTENTTYPE_TIME:
   case BHCELLCONTENTTYPE_DATETIME:
      break;
   }*/
}
/*
void BhEntryEditable::on_editing_done()
{
   entry->editing_done();
}
*/
/****************************************************************
 *
 *
 *
 *
 ****************************************************************/

BhCellRendererText::BhCellRendererText() : Glib::ObjectBase(typeid(BhCellRendererText))
{
   entry = Gtk::manage(new BhEntryEditable(""));
}

BhEntryEditable *BhCellRendererText::get_entry()
{
   return entry;
}

void BhCellRendererText::set(Glib::ustring str)
{
   entry->set_text(str);
}

void BhCellRendererText::clear()
{
}

Gtk::CellEditable *BhCellRendererText::start_editing_vfunc(GdkEvent *event, Gtk::Widget & widget, const Glib::ustring& path,
   const Gdk::Rectangle& background_area, const Gdk::Rectangle& cell_area, Gtk::CellRendererState flags)
{
   g_print("tonker\n");
   Gtk::CellEditable *ce = Gtk::CellRendererText::start_editing_vfunc(event,widget,path,background_area,cell_area,flags);
   if(!property_editable())
      return NULL;
   //BhEntryEditable *bhentry = Gtk::manage(new BhEntryEditable(path));
   Gtk::Entry *tentry = entry->get_entry();
   entry->grab_focus();
   Glib::ustring text = property_text();
   tentry->set_editable(true);
   tentry->set_text(text);
   tentry->select_region(0, 0);
   tentry->set_position(text.size());
   tentry->signal_editing_done().connect(bind(sigc::mem_fun(*this, &BhCellRendererText::on_edited), entry, path));
   tentry->show();
   return entry;
}

void BhCellRendererText::on_edited(BhEntryEditable *bhentry, Glib::ustring path)
{
   g_print("bonker\n");
   Gtk::Entry *entry = bhentry->get_entry();
   Glib::ustring old_text = property_text();
   Glib::ustring new_text = entry->get_text();
   if(new_text.compare(old_text)!=0){
      edited(path, new_text);
   }
}

/***************************************************
 *
 * BhBarcodeRenderer
 *
 ***************************************************/
void BhCellRendererBarcode::set(Glib::ustring str)
{
}

void BhCellRendererBarcode::clear()
{
}

Gtk::CellEditable *BhCellRendererBarcode::start_editing_vfunc(GdkEvent *event, Gtk::Widget & widget, const Glib::ustring& path,
   const Gdk::Rectangle& background_area, const Gdk::Rectangle& cell_area, Gtk::CellRendererState flags)
{
   if(!property_editable())
      return NULL;
   BhEntryEditable *bhentry = Gtk::manage(new BhEntryEditable(path));
   Gtk::Entry *entry = bhentry->get_entry();
   
   //BhEntry *entry = Gtk::manage(new BhEntry());
   Glib::ustring text = property_text();
   //g_print("errr: %s\n", text.data());
   //entry->SetType(TYPE_NUMERIC);
   //entry->SetMask("9999999999999");
   entry->set_editable(true);
   entry->set_text(text);
   entry->select_region(-1, -1);
   entry->set_position(text.size());
   entry->signal_editing_done().connect(bind(sigc::mem_fun(*this, &BhCellRendererBarcode::on_edited), bhentry, path));
   entry->show();
   return bhentry;
}

void BhCellRendererBarcode::on_edited(BhEntryEditable *bhentry, Glib::ustring path)
{
   Gtk::Entry *entry = bhentry->get_entry();
   Glib::ustring old_text = property_text();
   Glib::ustring new_text = entry->get_text();
   if(new_text.compare(old_text)!=0){
      edited(path, new_text);
   }
}

/***************************************************
 *
 * BhCombo
 *
 ***************************************************/
BhComboEditable::BhComboEditable(std::list<Glib::ustring> &items) : Glib::ObjectBase(typeid(BhComboEditable)),
   Gtk::EventBox(), Gtk::CellEditable()
{
   combo.get_entry()->set_has_frame(false);
   combo.set_popdown_strings(items);
   combo.show();
   add(combo);
}

BhComboEditable::BhComboEditable() : Glib::ObjectBase(typeid(BhComboEditable)),
   Gtk::EventBox(), Gtk::CellEditable()
{
   combo.get_entry()->set_has_frame(false);
   combo.show();
   add(combo);
}

Gtk::Entry *BhComboEditable::get_entry()
{
   return combo.get_entry();
}

void BhComboEditable::set_editable (bool editable)
{
   combo.get_entry()->set_editable(editable);
}

void BhComboEditable::start_editing_vfunc(GdkEvent *event)
{
   combo.get_entry()->start_editing(event);
}

void BhComboEditable::on_editing_done ()
{
   combo.get_entry()->editing_done();
}




void BhComboCellRenderer::add_item(Glib::ustring str)
{
   items.push_back(str);
}

void BhComboCellRenderer::clear()
{
   items.clear();
}

Gtk::CellEditable *BhComboCellRenderer::start_editing_vfunc(GdkEvent * event, Gtk::Widget & widget, const Glib::ustring& path,
   const Gdk::Rectangle & background_area, const Gdk::Rectangle & cell_area, Gtk::CellRendererState flags)
{
   if(!property_editable ())
      return NULL;
   BhComboEditable *combo = Gtk::manage(new BhComboEditable());
   Glib::ustring text = property_text();
   Gtk::Entry * entry = combo->get_entry();
   entry->property_editable() = true;
   entry->set_text(text);
   entry->select_region(0, -1);
   entry->signal_editing_done().connect(bind(sigc::mem_fun(*this, &BhComboCellRenderer::on_edited), entry, path));
   combo->show ();
   return combo;
}

void BhComboCellRenderer::on_edited (Gtk::Entry * entry, Glib::ustring path)
{
   Glib::ustring old_text = property_text();
   Glib::ustring new_text = entry->get_text();
   if(new_text.compare (old_text)!=0){
      edited (path, new_text);
   }
}

/*******************************************
void treeView_add_column(Gtk::TreeView& treeview,
const Glib::ustring& title, Gtk::CellRendererText*
render, Gtk::TreeModelColumn<ColumnType>&
model_column)
{
	int cols_count=treeview.append_column(title,
*render);
	Gtk::TreeViewColumn* pColumn =
treeview.get_column(cols_count-1);
	if(pColumn)
pColumn->add_attribute(render->property_text(),model_column);

Gtk::TreeView_Private::_connect_auto_store_editable_signal_handler(&treeview,render,model_column);
};
*******Usage*********************
CMy_Combo_CellRenderer* render0=Gtk::manage(new
CMy_Combo_CellRenderer());
render0->add_item("XYZ");
render0->add_item("XYZKMK");
treeView_add_column(m_list_table, "Students", render0,
m_list_columns.m_student_name);   
*/
  
}

}
