/*
bhcellrenderer.h
Copyright (C) 2004  Al Kingsley

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Part of:
	BhPos, Copyright (C) 2001-2004 Al Kingsley
	BhPos comes with ABSOLUTELY NO WARRANTY. 
*/
#ifndef _BHCELLRENDERER_H
#define _BHCELLRENDERER_H

#include <gtkmm/cellrenderer.h>
#include <gtkmm/cellrenderertext.h>
#include <gtkmm/eventbox.h>
#include <gtkmm/entry.h>
#include <gtkmm/combo.h>
#include "bhentry.h"

namespace bhpos{

/**
 * @defgroup bhwidget Widgets
 * @{
 *
 * @ingroup mflib
 * @{	
 */
   
namespace widget{

/*
 * @defgroup bhcellrenderer
 * @{
 *
 * @ingroup bhwidget
 * @{	
 */
 
enum BhCellType{
   BHCELLCONTENTTYPE_FIXED,
   BHCELLCONTENTTYPE_EDITABLE
};

/*
 *}@
 * @ingroup bhwidget
 * @{	
 */

enum BhCellContentType{
   BHCELLCONTENTTYPE_BARCODE,
   BHCELLCONTENTTYPE_TEXT,
   BHCELLCONTENTTYPE_INTEGER,
   BHCELLCONTENTTYPE_CURRENCY,
   BHCELLCONTENTTYPE_FLOAT,
   BHCELLCONTENTTYPE_COMBO,
   BHCELLCONTENTTYPE_PROGRESS,
   BHCELLCONTENTTYPE_DATE,
   BHCELLCONTENTTYPE_TIME,
   BHCELLCONTENTTYPE_DATETIME
};

/*
 *}@
 * @ingroup bhwidget
 * @{	
 */

// blah blah blah
class BhEntryEditable: public Gtk::EventBox, public Gtk::CellEditable
{
private:
   typedef BhEntryEditable Self;
   Glib::ustring path_;
   Gtk::Entry*   entry;
   bool          editing_canceled_;

   void on_entry_activate();
   bool on_entry_key_press_event(GdkEventKey* event);
   bool IsNumber(GdkEventKey *e);

protected:
   //virtual void start_editing_vfunc(GdkEvent *event);
   //virtual void on_editing_done();

   //virtual bool on_focus_out_event(GdkEventFocus *event);
   //virtual void on_changed(); 
   //virtual void on_activate(); 
   //virtual void on_insert_text(const Glib::ustring& text, int* position); //From Gtk::Editable
   //virtual void check_for_change();
    
   //type_signal_edited m_signal_edited;

   virtual bool on_key_press_event(GdkEventKey* event);
   virtual void start_editing_vfunc(GdkEvent* event);

   BhCellContentType  type;
   //BhEntry           *entry;
   //Glib::ustring      old_text;

public:
   BhEntryEditable(const Glib::ustring& path);
   Gtk::Entry *get_entry() { return entry; }
   Glib::PropertyProxy_Base _property_renderable();
   Glib::ustring get_path() const;
   void set_text(const Glib::ustring& text);
   Glib::ustring get_text() const;
   void select_region(int start_pos, int end_pos);
   bool get_editing_canceled() const;
   void set_position(int);
   int  get_text_length();
/*
   void set_editable(bool editable);
   void set_type(BhCellContentType t) { type=t; }
   void set_text(Glib::ustring);
   Glib::ustring get_text();
   void select_region(int, int);
   */
};

/*
 *}@
 * @ingroup bhwidget
 * @{	
 */

class BhCellRendererText : public Gtk::CellRendererText
{
private:
   typedef BhCellRendererText  Self;
   BhEntryEditable            *entry;
   bool on_button_press_event(GdkEventButton* event);
   bool on_key_press_event(GdkEventKey* event);

protected:
   virtual Gtk::CellEditable *start_editing_vfunc(GdkEvent *, Gtk::Widget&, 
      const Glib::ustring&, const Gdk::Rectangle&, const Gdk::Rectangle &, Gtk::CellRendererState flags);
   virtual void on_edited(BhEntryEditable *entry, Glib::ustring path);

public:
   BhCellRendererText();
   BhEntryEditable *get_entry();
   void set(Glib::ustring str);
   void clear();
};

/*
 *}@
 * @ingroup bhwidget
 * @{	
 */

class BhCellRendererBarcode : public Gtk::CellRendererText
{
private:
   typedef BhCellRendererBarcode  Self;
   BhEntryEditable               *entry;

protected:
   virtual Gtk::CellEditable *start_editing_vfunc (GdkEvent *, Gtk::Widget&, 
      const Glib::ustring&, const Gdk::Rectangle&, const Gdk::Rectangle &, Gtk::CellRendererState flags);
   virtual void on_edited(BhEntryEditable *entry, Glib::ustring path);

public:
   void set(Glib::ustring str);
   void clear();
};

/*
 *}@
 * @ingroup bhwidget
 * @{	
 */

class BhComboEditable: public Gtk::EventBox, public Gtk::CellEditable
{
private:
   Gtk::Combo combo;

protected:
   virtual void start_editing_vfunc(GdkEvent *event);
   virtual void on_editing_done();

public:
   BhComboEditable();
   BhComboEditable(std::list<Glib::ustring> &items);
   Gtk::Entry *get_entry();
   void set_editable(bool editable);
};

/*
 *}@
 * @ingroup bhwidget
 * @{	
 */

class BhComboCellRenderer : public Gtk::CellRendererText
{
public:
   void add_item(Glib::ustring str);
   void clear();

private:
   std::list<std::string> items;

protected:
   virtual Gtk::CellEditable *start_editing_vfunc(GdkEvent *, Gtk::Widget&, 
      const Glib::ustring&, const Gdk::Rectangle&, const Gdk::Rectangle &, Gtk::CellRendererState flags);
   virtual void on_edited(Gtk::Entry *entry, Glib::ustring path);
};

/* }@ 
	}@ */

}

/* }@ 
	}@ */

}

#endif   // _BHCELLRENDERER_H_
