/* bhentry.h
 *
 * Copyright (C) 2004 Al Kingsley
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef BHENTRY_H
#define BHENTRY_H

#include <clocale>     // for locale, time_put
#include <gtkmm/entry.h>

namespace bhpos{

/**
 * @defgroup bhwidget Widgets
 * @{
 *
 * @ingroup mflib
 * @{	
 */
   
namespace widget{

/*
 * @defgroup bhentry
 * @{
 *
 * @ingroup bhwidget
 * @{	
 */

class BhEntry : public Gtk::Entry
{
public:
   enum bhentry_type
   {
      TYPE_NONE=0,
      TYPE_NUMERIC,
      TYPE_CURRENCY,
      TYPE_TEXT,
      TYPE_DATE,
      TYPE_TIME,
      TYPE_DATETIME,
      TYPE_BOOLEAN
   };
   BhEntry();
   BhEntry(BaseObjectType* cobject);
   BhEntry(bhentry_type t);
   virtual ~BhEntry();

   void SetType(bhentry_type t) { type= t; }
   void SetMask(Glib::ustring m) { mask=m; }
   void SetFormat(Glib::ustring f) { format=f; }
   void SetColour(Gdk::Color);
   
   Gtk::Entry::set_max_length;
   Gtk::Entry::set_has_frame;
   Gtk::Entry::set_activates_default;
   Gtk::Entry::set_visibility;
   Gtk::Entry::get_text;
   Gtk::Entry::set_editable;
   
   Glib::ustring GetMask() { return mask; }
   Glib::ustring GetFormat() { return format; }
   
   //void set_text(const Glib::ustring& text); //override
   typedef sigc::signal<void> type_signal_edited;
   type_signal_edited getsignaledited();

protected:
   //Overrides of default signal handlers:
   virtual void on_changed(); //From Gtk::Entry.
   virtual void on_activate(); //From Gtk::Entry.
   virtual bool on_focus_out_event(GdkEventFocus* event); //From Gtk::Widget
   //virtual void on_insert_text(const Glib::ustring& text, int* position); //From Gtk::Editable

   virtual void check_for_change();
    
   //bool CheckText(Glib::ustring new_text);

   type_signal_edited signal_edited;

   lconv *lc;
   Glib::ustring old_text;
   bhentry_type  type; //Store the type so we can validate the text accordingly.
   Glib::ustring mask;
   Glib::ustring format;
};


/* }@ 
	}@ */

}

/* }@ 
	}@ */

}

#endif // BHENTRY_H
